@@ -204,11 +204,14 @@ def execute(task, *args, **kwargs):
                     # if it can't.
                     try:
                         import multiprocessing
-                    except ImportError, e:
-                        msg = "At least one task needs to be run in parallel, but the\nmultiprocessing module cannot be imported:"
-                        msg += "\n\n\t%s\n\n" % e
-                        msg += "Please make sure the module is installed or that the above ImportError is\nfixed."
-                        abort(msg)
+                    except ImportError:
+                        import traceback
+                        tb = traceback.format_exc()
+                        abort(tb + """
+At least one task needs to be run in parallel, but the
+multiprocessing module cannot be imported (see above
+traceback.) Please make sure the module is installed
+or that the above ImportError is fixed.""")
 
                     # Wrap in another callable that nukes the child's cached
                     # connection object, if needed, to prevent shared-socket
@@ -231,7 +234,14 @@ def execute(task, *args, **kwargs):
         # If running in parallel, block until job queue is emptied
         if jobs:
             jobs.close()
-            jobs.start()
+            exitcodes = jobs.run()
+            # Abort if any children did not exit cleanly (fail-fast).
+            # This prevents Fabric from continuing on to any other tasks.
+            if any([x != 0 for x in exitcodes]):
+                abort("One or more hosts failed while executing task '%s'" % (
+                    my_env['command']
+                ))
+
     # Or just run once for local-only
     else:
         with settings(**my_env):
