@@ -10,7 +10,7 @@ from subprocess import Popen, PIPE
 from os.path import abspath, dirname
 
 
-VERSION = (1, 3, 1, 'final', 0)
+VERSION = (1, 3, 3, 'final', 0)
 
 
 def git_sha():
@@ -39,6 +39,7 @@ def get_version(form='short'):
     * ``verbose``: like ``normal`` but fully explicit, e.g. "0.9 final". For
       tag commit messages, or anywhere that it's important to remove ambiguity
       between a branch and the first final release within that branch.
+    * ``all``: Returns all of the above, as a dict.
     """
     # Setup
     versions = {}
@@ -93,6 +94,11 @@ def get_version(form='short'):
     try:
         return versions[form]
     except KeyError:
+        if form == 'all':
+            return versions
         raise TypeError('"%s" is not a valid form specifier.' % form)
 
 __version__ = get_version('short')
+
+if __name__ == "__main__":
+    print get_version('all')
